package com.usky.oa.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.oa.domain.*;
import com.usky.oa.mapper.*;
import com.usky.oa.service.OaApprovalService;
import com.usky.oa.service.OaFbdDocumentService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.oa.service.utils.OaApprovalGeneration;
import com.usky.oa.service.utils.OaSendMessageCenter;
import com.usky.system.domain.SysUser;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 发布申请表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2025-02-20
 */
@Service
public class OaFbdDocumentServiceImpl extends AbstractCrudService<OaFbdDocumentMapper, OaFbdDocument> implements OaFbdDocumentService {

    @Autowired
    private OaFormDefinitionMapper oaFormDefinitionMapper;

    @Autowired
    private OaDocumentMapper oaDocumentMapper;

    @Autowired
    private OaFbdDocumentMapper oaFbdDocumentMapper;

    @Autowired
    private OaApprovalMapper oaApprovalMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private SysUserPostMapper sysUserPostMapper;

    @Autowired
    private OaApprovalGeneration oaApprovalGeneration;

    @Autowired
    private OaSendMessageCenter oaSendMessageCenter;

    @Autowired
    private OaApprovalService oaApprovalService;

    @Override
    public void add(OaFbdDocument oaFbdDocument) {
        // 校验表单数据
        validateDocument(oaFbdDocument);

        Long userId = SecurityUtils.getUserId();
        String username = SecurityUtils.getUsername();
        Long deptId = SecurityUtils.getLoginUser().getSysUser().getDeptId();
        Integer tenantId = SecurityUtils.getTenantId();

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyMMddHHmmss");
        LocalDateTime now = LocalDateTime.now();
        String formattedDate = now.format(formatter);

        LambdaQueryWrapper<OaFormDefinition> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(OaFormDefinition::getFormSign).eq(OaFormDefinition::getId, oaFbdDocument.getFormId());
        OaFormDefinition oaFormDefinition = oaFormDefinitionMapper.selectOne(queryWrapper);
        String sign = oaFormDefinition.getFormSign();
        String docNo = sign + "-" + formattedDate;

        // 插入发布表数据
        oaFbdDocument.setDocNo(docNo);
        oaFbdDocument.setProposer(userId);
        oaFbdDocument.setCreateBy(username);
        oaFbdDocument.setCreateTime(now);
        oaFbdDocument.setDeptId(deptId);
        oaFbdDocument.setTenantId(tenantId);
        oaFbdDocumentMapper.insert(oaFbdDocument);

        // 插入单据表数据
        OaDocument oaDocument = new OaDocument();
        oaDocument.setFormId(oaFbdDocument.getFormId());
        oaDocument.setType(sign);
        oaDocument.setDocNo(docNo);
        oaDocument.setProposer(userId);
        oaDocument.setDocStatus(oaFbdDocument.getDocStatus());
        oaDocument.setCreateBy(username);
        oaDocument.setCreateTime(oaFbdDocument.getCreateTime());
        oaDocument.setDeptId(deptId);
        oaDocument.setTenantId(tenantId);
        oaDocumentMapper.insert(oaDocument);

        // 插入审批表数据,第一条审批记录
        if (oaFbdDocument.getDocStatus().equals(1)) {
            getOaApproval(oaFbdDocument);
        }
    }

    // 更新发布表数据
    //@Transactional(rollbackFor = Exception.class)
    @Transactional
    @Override
    public void update(OaFbdDocument oaFbdDocument) {
        // 验证文档
        validateDocument(oaFbdDocument);

        // 获取当前用户和时间
        String username = SecurityUtils.getUsername();
        LocalDateTime now = LocalDateTime.now();
        Integer tenantId = SecurityUtils.getTenantId();

        // 设置更新信息
        oaFbdDocument.setUpdateBy(username);
        oaFbdDocument.setUpdateTime(now);

        // 更新发布表数据
        oaFbdDocumentMapper.updateById(oaFbdDocument);

        // 更新总单据状态
        oaDocumentMapper.update(null, Wrappers.lambdaUpdate(OaDocument.class)
                .eq(OaDocument::getDocNo, oaFbdDocument.getDocNo())
                .set(OaDocument::getDocStatus, oaFbdDocument.getDocStatus())
                .set(OaDocument::getUpdateBy, username)
                .set(OaDocument::getUpdateTime, now)
        );

        // 如果文档状态为1，创建并插入审批记录
        if (oaFbdDocument.getDocStatus() == 1) {
            OaFbdDocument updatedDocument = oaFbdDocumentMapper.selectById(oaFbdDocument.getId());
            getOaApproval(updatedDocument);
        }
    }

    // 获取审批表数据
    @Override
    public void getOaApproval(OaFbdDocument oaFbdDocument) {
        String docNo = oaFbdDocument.getDocNo();
        String formSign = docNo.split("-")[0];
        OaFlow oaFlow = oaApprovalGeneration.selectBySign(formSign);
        OaNode oaNode = oaApprovalGeneration.selectByFlowId(oaFlow.getId());

        List<Long> userIds = new ArrayList<>();

        switch (oaNode.getNodeScope()) {
            case 0:
            case 1:
            case 2:
                userIds = Arrays.stream(oaNode.getProposer().split(",")).map(Long::parseLong).collect(Collectors.toList());
                break;
            case 3:
                LambdaQueryWrapper<SysUser> userQuery = new LambdaQueryWrapper<>();
                userQuery.select(SysUser::getUserId)
                        .eq(SysUser::getTenantId, oaFlow.getTenantId())
                        .eq(SysUser::getDelFlag, 0)
                        .eq(SysUser::getStatus, 0)
                        .eq(SysUser::getDeptId, oaNode.getDeptId());
                userIds = sysUserMapper.selectList(userQuery).stream().map(SysUser::getUserId).collect(Collectors.toList());
                break;
            case 4:
                LambdaQueryWrapper<SysUserPost> userRoleQuery = new LambdaQueryWrapper<>();
                userRoleQuery.eq(SysUserPost::getPostId, oaNode.getJobId());
                userIds = sysUserPostMapper.selectList(userRoleQuery).stream().map(SysUserPost::getUserId).collect(Collectors.toList());
                break;
        }

        for (Long userId1 : userIds) {
            OaApproval approval = new OaApproval();
            approval.setDocNo(docNo);
            approval.setProposer(Math.toIntExact(oaFbdDocument.getProposer()));
            approval.setType(oaNode.getNodeType());
            approval.setApprovalStatus(1);
            approval.setApprovalUid(userId1);
            approval.setSubmitDate(oaFbdDocument.getCreateTime());
            approval.setCreateTime(oaFbdDocument.getCreateTime());
            approval.setCreateBy(oaFbdDocument.getCreateBy());
            approval.setDeptId(oaFbdDocument.getDeptId());
            approval.setTenantId(oaFbdDocument.getTenantId());
            approval.setNodeId(oaNode.getId());
            oaApprovalMapper.insert(approval);
        }

        oaSendMessageCenter.sendAsyncMessage(
                oaFbdDocument.getCreateBy(),
                oaFbdDocument.getProposer(),
                oaFbdDocument.getId(),
                userIds,
                3,
                oaApprovalService.getFieldInfo(oaFbdDocument.getDocNo())
        );
    }

    /**
     * 校验表单数据
     * @param oaFbdDocument
     */
    private void validateDocument(OaFbdDocument oaFbdDocument) {
        if (StringUtils.isBlank(oaFbdDocument.getApplicationName())) {
            throw new BusinessException("申请名称不能为空！");
        } else if (oaFbdDocument.getApplicationName().length() > 16) {
            throw new BusinessException("申请名称长度不能超过16个字符!");
        } else if (oaFbdDocument.getPublishTime() == null) {
            throw new BusinessException("发布时间不能为空！");
        } else if (oaFbdDocument.getPublishTime().isBefore(LocalDateTime.now())) {
            throw new BusinessException("发布时间不能早于当前时间！");
        } else if (oaFbdDocument.getApplicationName().length() > 15) {
            throw new BusinessException("申请名称长度不能超过15个字符!");
        } else if (StringUtils.isBlank(oaFbdDocument.getPublishProject())) {
            throw new BusinessException("发布项目不能为空！");
        } else if (oaFbdDocument.getPublishProject().length() > 64) {
            throw new BusinessException("发布项目名称长度不能超过64个字符!");
        } else if (oaFbdDocument.getProjectType() == null) {
            throw new BusinessException("项目类型不能为空!");
        } else if (oaFbdDocument.getProjectType() < 1 || oaFbdDocument.getProjectType() > 2) {
            throw new BusinessException("项目类型传参有误!");
        } else if (StringUtils.isBlank(oaFbdDocument.getOnlineFunction())) {
            throw new BusinessException("上线功能不能为空!");
        } else if (oaFbdDocument.getOnlineFunction().length() > 250) {
            throw new BusinessException("上线功能长度不能超过250个字!");
        } else if (oaFbdDocument.getPreparation() != null && oaFbdDocument.getPreparation().length() > 64) {
            throw new BusinessException("上线前准备不能超过250个字!");
        } else if (oaFbdDocument.getAnnex() != null && oaFbdDocument.getAnnex().length() > 500) {
            throw new BusinessException("附件上传数量超!");
        }
    }


}
